/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2003/12/23 12:57:28 $ $Revision: 1.22 $ */

#ifndef BODY_H
#define BODY_H

#define BT_EOL		0
#define BT_Wall		1
#define BT_Ball		2
#define BT_Cylinder	3
#define BT_Disk		4
#define BT_BallCC	5
#define BT_Force	6
#define BT_STBall	7
#define BT_DTEdge	8
#define BT_DGBall	9
#define BT_DGEllipsoid	10
#define BT_XYZBall	11

/* body parameter (type 4) */
#define BP_Mask		31
#define BP_Pos		32
#define BP_Orient	64
#define BP_Coord	128
#define BP_Velo		256
#define BP_Force	512
#define BP_AngVelo	1024
#define BP_Torque	2048
#define BP_TrVelo	4096
#define BP_TrForce	8192
#define BP_TrAngVelo	16384


#define BA_Texture1D	1
#define BA_Texture2D	2
#define BA_Color	4
#define BA_Blend	8

#define CEF_PARNUM	12	/* number of parameter types */
#define CEF_SUBPARNUM	5	/* number of subparameters */

/* objects */
typedef struct BODY_ {
  int type;		/* type flag  BT_... */
  int obj;		/* object group it belongs to */
  int par;		/* parameters present BP_... */
  int atom;		/* Atom type in xyz format */
  double x,y,z;		/* position */
  double rx,ry,rz;	/* rotation diretciont */
  double phi;		/* rotation angle in degrees */
  double nx,ny,nz;	/* normal vector */
  double sx,sy,sz;	/* size */
  int appearance;	/* texture, color, material BA_... */
  int texture;		/* texture */
  GLenum material;	/* colormaterial type eg. GL_DIFFUSE */
  int ind1,ind2;	/* force index parameter */
  double params[CEF_PARNUM*CEF_SUBPARNUM+1];	/* parameters */
} BODY;

BODY *body;

typedef struct CONSTRAINT_ {
  struct CONSTRAINT_ *next;	/* next CONSTRAINT */
  int param;			/* chosen param */
  int rel;			/* relation */
  double val;			/* value */
  int strrel;			/* string relation */
  char *str;			/* string to be used if not NULL */
  RPN *rpnroot;			/* rpn root for string */
} CONSTRAINT;

typedef struct bodydisplaystr_ {
  int show;		/* 1-> show this type of object */
  int blend;		/* 1-> this object is transparent */
  int texture;		/* texture number+1, 0-> no texture */
  int type;		/* type of object */
  int par;		/* parameters of the object */
  int atom;		/* Atom type in xyz format */
  GLfloat color[4];	/* color if no texture */
  int scale_force;	/* scale force to maximum? */
  double fcalscale;	/* scale calculated from the max */
  double fscale;	/* multiply force by this value */
  double fsize;		/* size of maximal force */
  int constr_num;	/* number of constraints */
  CONSTRAINT *constr;	/* constraints */
  CONSTRAINT *newconstr;/* new constraints */
} bodydisplaystr;

typedef struct bodyglobal_ {
  int body_num;			/* actual number of bodies */
  int wall_num;			/* actual number of bodies */
  int input_type;		/* input file structure */
  int line_length;		/* line length */
  int active_obj;		/* subframe's object */
  int obj_num;			/* number of different objects */
  int xyz_num;			/* number of xyz atoms */
  bodydisplaystr *param;	/* show object type */
  GLfloat cc[4];		/* calculated color if color scale is used */
  double fdx;			/* calculated size factor */
  double arrowsize;		/* size of the arrow */
  double arrowwidth;		/* width of the arrow */
  int arrowdir;			/* direction of the arrow, param number */
  CONSTRAINT *ac;		/* active constraint */
  GtkWidget *textmenu;		/* texture menu widget */
  GtkWidget *frame;		/* frame for properties */
  GtkWidget *forcehbox;		/* parameters only for forces */
  GtkWidget *blend;		/* blending chk button */
  GtkWidget *texarea;		/* widget area for texture */
  GtkWidget *r,*g,*b,*a;	/* widgets for colors */
  GtkWidget *forcescale;	/* scale force to maximum? */
  GtkWidget *fscalelabel;	/* label of force scale value */
  GtkWidget *fscalevalue;	/* widget to enter force scale */
  GtkWidget *constraintbutt;	/* button for constraint frame */
  GtkWidget *constframe;	/* constraints frame */
  GtkWidget *constnum;		/* constraints number */
  GtkWidget *constpar;		/* constraints parameter */
  GtkWidget *constparlabel;	/* constraints parameter label */
  GtkWidget *constrel;		/* constraints relation */
  GtkWidget *constval;		/* constraints value */
  GtkWidget *constdel;		/* constraints delete */
  GtkWidget *constadd;		/* constraints ok */
  GtkWidget *constnummenu;	/* constraints number menu */
  GtkWidget *conststrmenu;	/* constraints string menu*/
  GtkWidget *conststr;		/* constraints string */
  int constrnummenusize;	/* constraints number menu size */
  GtkWidget **constrnummenuitem;/* menu items of latter */
  GtkWidget *parmainitem[CEF_PARNUM];
  				/* menu for parameter in constraints */
} bodyglobal;

bodyglobal bg;

#define XYZ_Known_Atoms 15
extern const char cef_relstr[6][2];
extern const char cef_parmainstr[CEF_PARNUM][10];
extern const char cef_parstr[CEF_PARNUM][CEF_SUBPARNUM][4];
extern const char XYZ_Known_Atoms_s[XYZ_Known_Atoms][3];
extern const double XYZ_Atom_size[XYZ_Known_Atoms];

#endif
