#include <ft2build.h>
#include <freetype/freetype.h>
#include <freetype/ftglyph.h>
#include <freetype/ftoutln.h>
#include <freetype/fttrigon.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include "font.h"
#include "global.h"

/* Prototypes */
void perr(char *, ...);

inline int next_p2 ( int a )
{
  int rval=1;
  while(rval<a) rval<<=1;
  return rval;
}


void make_dlist(FT_Face face,int ch,GLuint list_base,GLuint *tex_base)
{
  FT_Glyph glyph;
  FT_BitmapGlyph bitmap_glyph;
  FT_Bitmap bitmap;
  int width,height,i,j;
  GLubyte *expanded_data;
  double x,y;

  if (FT_Load_Glyph(face,FT_Get_Char_Index(face,ch),FT_LOAD_DEFAULT)) {
    perr("FT_Load_Glyph failed\n");
  }
  if (FT_Get_Glyph(face->glyph,&glyph)) perr("FT_Get_Glyph failed\n");
  FT_Glyph_To_Bitmap(&glyph,ft_render_mode_normal,0,1);
  bitmap_glyph=(FT_BitmapGlyph) glyph;
  bitmap=bitmap_glyph->bitmap;
  width=next_p2(bitmap.width);
  height=next_p2(bitmap.rows);
  expanded_data=(GLubyte *)malloc(2*width*height*sizeof(GLubyte));
  for (j=0; j <height;j++) {
    for (i=0; i < width; i++) {
      expanded_data[2*(i+j*width)]= expanded_data[2*(i+j*width)+1] =
        (i>=bitmap.width || j>=bitmap.rows) ?
        0 : bitmap.buffer[i + bitmap.width*j];
    }
  }
  glBindTexture( GL_TEXTURE_2D, tex_base[ch]);
  glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, width, height,
      0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, expanded_data );
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
  free(expanded_data);

  glNewList(font.list_base+ch,GL_COMPILE);
  glBindTexture(GL_TEXTURE_2D,tex_base[ch]);
  glTranslatef(bitmap_glyph->left,0,0);
  glPushMatrix();
  glTranslatef(0,bitmap_glyph->top-bitmap.rows,0);
  x=(double)bitmap.width / (double)width,
  y=(double)bitmap.rows / (double)height;
  glBegin(GL_QUADS);
  glTexCoord2d(0,0); glVertex2f(0,bitmap.rows);
  glTexCoord2d(0,y); glVertex2f(0,0);
  glTexCoord2d(x,y); glVertex2f(bitmap.width,0);
  glTexCoord2d(x,0); glVertex2f(bitmap.width,bitmap.rows);
  glEnd();
  glPopMatrix();
  glTranslatef(face->glyph->advance.x >> 6 ,0,0);
  glEndList();
}



void font_init(const char * fname, unsigned int h)
{
  FT_Library library;
  FT_Face face;
  int i;

  font.h=(double)h;
  if (FT_Init_FreeType(&library)) perr("FT_Init_FreeType failed\n");
  if (FT_New_Face(library,fname,0,&face))  {
    perr("FT_New_Face failed (probably a problem with your font file)\n");
  }
  FT_Set_Char_Size(face,h << 6,h << 6,96,96);
  font.list_base=glGenLists(128);
  glGenTextures(128,font.textures);
  for(i=0;i<128;i++) {
    make_dlist(face,i,font.list_base,font.textures);
  }
  FT_Done_Face(face);
  FT_Done_FreeType(library);
}

/*void font_data::clean() {
	glDeleteLists(font.list_base,128);
	glDeleteTextures(128,font.textures);
	delete [] textures;
}*/




inline void pushScreenCoordinateMatrix()
{
  GLint  viewport[4];

  glPushAttrib(GL_TRANSFORM_BIT);
  glGetIntegerv(GL_VIEWPORT, viewport);
  glMatrixMode(GL_PROJECTION);
  glPushMatrix();
  glLoadIdentity();
  gluOrtho2D(viewport[0],viewport[2],viewport[1],viewport[3]);
  glPopAttrib();
}

void pop_projection_matrix()
{
  glPushAttrib(GL_TRANSFORM_BIT);
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glPopAttrib();
}

void print_GL_font(double x,double y,const char *fmt, ...)
{
  GLuint f;
  double h;
  char text[1000];
  va_list ap;
  GLfloat modelview_matrix[16];

  pushScreenCoordinateMatrix();
  f=font.list_base;
  h=font.h/.63f;
  if (fmt == NULL) return;
  va_start(ap, fmt);
  vsprintf(text, fmt, ap);
  va_end(ap);
  glPushAttrib(GL_LIST_BIT | GL_CURRENT_BIT  | GL_ENABLE_BIT |
      						GL_TRANSFORM_BIT);
  glMatrixMode(GL_MODELVIEW);
  glDisable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);
  glDisable(GL_DEPTH_TEST);
  glEnable(GL_BLEND);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glListBase(f);
  glGetFloatv(GL_MODELVIEW_MATRIX, modelview_matrix);

  glPushMatrix();
  glLoadIdentity();
  glTranslatef(x,y,0);
  glMultMatrixf(modelview_matrix);
  glCallLists(strlen(text), GL_UNSIGNED_BYTE, text);

  glPopMatrix();

  glPopAttrib();
  pop_projection_matrix();
}

