/* Copyright (C) 2002, by Sandor Fazekas and Janos Torok */
/* Last modified: $Date: 2003/11/13 15:49:35 $ $Revision: 1.9 $ */
/* gtk menu with itemfactory */
#include <gtk/gtk.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "global.h"
#include "menu.h"
#include "anim.h"
#ifndef _WIN32
#include <unistd.h>
#endif
#ifdef _WIN32
#include <process.h>
#endif

/* prototypes */
gint Quit(GtkWidget *);
gint AnimateBody(gpointer);
int ilog(double);
void Change_FramedtMax(double);
void Pause_Signal(GtkWidget *,gpointer);
void Set_PlayDir(GtkWidget *,gpointer);
void Snapshot(GtkWidget *,gpointer);
void SaveSnap(GtkWidget *,gpointer);
void About(void);
void Help(void);
void Write_rc(void);

void Destroy_FileOpen(GtkWidget *widget,gpointer data)
{
  gtk_widget_destroy(GTK_WIDGET(data));
  menuvars.openfileisup=0;
}

void Change_Process(GtkWidget *widget,gpointer data)
{
  execlp(argv0,argv0,
      gtk_file_selection_get_filename( GTK_FILE_SELECTION(data)), NULL);
}

void OpenNew(void)
{
  GtkWidget *filesel;

  if (menuvars.openfileisup) return;
  menuvars.openfileisup=1;
  filesel=gtk_file_selection_new("Open file");
  gtk_signal_connect (GTK_OBJECT (filesel), "delete_event",
	      GTK_SIGNAL_FUNC (Destroy_FileOpen), (gpointer)filesel );
  gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION(filesel)->cancel_button),
	      "clicked",GTK_SIGNAL_FUNC (Destroy_FileOpen), (gpointer)filesel );
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(filesel)->ok_button),
		"clicked",GTK_SIGNAL_FUNC (Change_Process),(gpointer)filesel );
  gtk_widget_show(filesel);
}

static void Menu_Quit(GtkWidget *widget, gpointer data)
{
  Quit(widget);
}

void Menu_Run(GtkWidget *widget, gpointer data)
{
  long a;
  a=(long) data;
  if (a) { /* stop */
    if (anim.timeouttag) gtk_timeout_remove(anim.timeouttag);
    anim.timeouttag=0;
    anim.running=0;
  } else if (!anim.running) { /* run */
    if (!anim.timeouttag) {
      anim.timeouttag=gtk_timeout_add(anim.framedt,AnimateBody,
	    GTK_OBJECT(gl.glarea));
    }
    anim.running=1;
    anim.imeasuredt=0;
  }
}

void Menu_Edit_Destroy(GtkWidget *widget, gpointer data)
{
  long a;

  a=(long)data;

  gtk_widget_destroy(GTK_WIDGET(menuvars.entry_win_widget[a]));
  menuvars.entry_open[a]=0;
}

void Menu_Edit_Set_Value(GtkWidget *widget, gpointer data)
{
  long a;

  a=(long)data;
  switch (a) {
    case 0:	/* set dt max */
      Change_FramedtMax(
	atof(gtk_entry_get_text(GTK_ENTRY(menuvars.entry_widget[a]))));
      /* XXX check length */
      break;
    case 1:	/* set fast forward multiplier */
      animation.ffval=atoi(gtk_entry_get_text(
	    		GTK_ENTRY(menuvars.entry_widget[a])));
      break;
    case 2:	/* set XYZ factor */
      animation.xyzf=atof(gtk_entry_get_text(
	      GTK_ENTRY(menuvars.entry_widget[a])));
      break;
    case 3:	/* set sphere min resolution for adjustment */
      animation.quadresmin=min(999,atoi(gtk_entry_get_text(
	    			GTK_ENTRY(menuvars.entry_widget[a]))));
      break;
    case 4:	/* set framedt limit for sphere resolution decrese */
      animation.quadadjustdtlim=atoi(gtk_entry_get_text(
	    			GTK_ENTRY(menuvars.entry_widget[a])));
      break;
    case 5:	/* set factor */
      gl.f=atof(gtk_entry_get_text(
	    			GTK_ENTRY(menuvars.entry_widget[a])));
      break;
    case 6:	/* set key factor */
      anim.keyf=atof(gtk_entry_get_text(
	      GTK_ENTRY(menuvars.entry_widget[a])));
      break;
  }
  gtk_widget_destroy(GTK_WIDGET(menuvars.entry_win_widget[a]));
  menuvars.entry_open[a]=0;
}

void Menu_Edit(GtkWidget *widget, gpointer data)
{
  GtkWidget *entry_win;		/* Create edit win */
  GtkWidget *vbox;		/* vertical box */
  GtkWidget *hbox1,*hbox2;	/* horisontal boxes upper,lower */
  GtkWidget *entry;		/* entry widget */
  GtkWidget *msg;		/* message widget */
  GtkWidget *ok;		/* ok button */
  GtkWidget *cancel;		/* cancel button */
  static char *text[]= { "Frame dt maximum",
  			 "Fast play multiplier",
  			 "XYZ factor",
  			 "Min sphere resolution in adjustment",
  			 "Frame dt limit for sphere resolution adjustment",
  			 "Enlargement factor",
  			 "Step factor in key movement" };
  long a;
  int l,num,v;
  double w;
  char *s=NULL;

  a=(long)data;
  if (menuvars.entry_open[a]) return;
  entry_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  menuvars.entry_win_widget[a]=entry_win;
  menuvars.entry_open[a]=1;
  gtk_widget_set_usize( GTK_WIDGET (entry_win), 350, 70);
  gtk_window_set_title(GTK_WINDOW (entry_win), text[a]);
  vbox=gtk_vbox_new(TRUE,0);
  hbox1=gtk_hbox_new(FALSE,0);
  hbox2=gtk_hbox_new(FALSE,0);
  gtk_container_add(GTK_CONTAINER(entry_win), vbox);
  gtk_box_pack_start (GTK_BOX (vbox), hbox1, TRUE, TRUE, 5);
  gtk_box_pack_start (GTK_BOX (vbox), hbox2, TRUE, TRUE, 5);
  ok = gtk_button_new_with_label ("ok");
  cancel = gtk_button_new_with_label ("cancel");
  gtk_box_pack_start(GTK_BOX(hbox2), ok, TRUE, TRUE, 10);
  gtk_box_pack_start(GTK_BOX(hbox2), cancel, TRUE, TRUE, 10);
  entry = gtk_entry_new();
  menuvars.entry_widget[a]=entry;
  gtk_widget_set_usize( GTK_WIDGET (entry), 50, 0);
  msg = gtk_label_new(text[a]);
  num=1;
  switch (a) {
    case 0:	/* set framedt max */
      l=ilog(GTK_ADJUSTMENT(anim.animframeslideradj)->upper);
      v=(int)(GTK_ADJUSTMENT(anim.animframeslideradj)->upper);
      break;
    case 1:	/* set fast forward multiplier */
      l=max(5,ilog((double)animation.ffval));
      v=animation.ffval;
      break;
    case 2:	/* set XYZ factor */
      l=max(5,ilog((double)animation.xyzf));
      v=animation.xyzf;
      break;
    case 3:	/* set sphere min resolution for adjustment */
      l=max(5,ilog((double)animation.quadresmin));
      v=animation.quadresmin;
      break;
    case 4:	/* set framedt limit for sphere resolution decrese */
      l=max(5,ilog((double)animation.quadadjustdtlim));
      v=animation.quadadjustdtlim;
      break;
    case 5:	/* factor */
      num=2;
      l=max(5,ilog(gl.f)+3);
      w=gl.f;
      break;
    case 6:	/* set key factor */
      l=max(5,ilog((double)anim.keyf));
      v=anim.keyf;
      break;
  }
  if (num) {
    s=(char *) calloc(l+1,sizeof(char));
    if (num==1) sprintf(s,"%d",v);
    else sprintf(s,"%.2lf",w);
    gtk_entry_set_text(GTK_ENTRY(entry),s);
    free(s);
  } else {
    gtk_entry_set_text(GTK_ENTRY(entry),s);
    free(s);
  }
  gtk_entry_select_region (GTK_ENTRY (entry),0,GTK_ENTRY(entry)->text_length);
  gtk_box_pack_start (GTK_BOX (hbox1), msg, FALSE, FALSE, 5);
  gtk_box_pack_start (GTK_BOX (hbox1), entry, TRUE, TRUE, 5);
  gtk_widget_show (ok);
  gtk_widget_show (cancel);
  gtk_widget_show (entry);
  gtk_widget_show (msg);
  gtk_widget_show (vbox);
  gtk_widget_show (hbox1);
  gtk_widget_show (hbox2);
  gtk_widget_show (entry_win);

  gtk_signal_connect(GTK_OBJECT (entry), "activate",
                       (GtkSignalFunc) Menu_Edit_Set_Value, data);
  gtk_signal_connect(GTK_OBJECT (entry_win), "delete_event",
                       (GtkSignalFunc) Menu_Edit_Destroy, data);
  gtk_signal_connect(GTK_OBJECT (ok), "clicked",
                       (GtkSignalFunc) Menu_Edit_Set_Value, data);
  gtk_signal_connect(GTK_OBJECT (cancel), "clicked",
                       (GtkSignalFunc) Menu_Edit_Destroy, data);
}

void Menu_Frame(GtkWidget *widget, gpointer data)
{
  int a;

  a=(int)data;
  switch (a) {
    case 0: /* Frame frequency frame */
      gtk_widget_show(menuvars.ffframe);
      break;
    case 1: /* Movie frame */
      gtk_widget_show(menuvars.movieframe);
      break;
    case 2: /* Eyeframe */
      gtk_widget_show(menuvars.eyeframe);
      break;
    case 3: /* Orientationframe */
      gtk_widget_show(menuvars.orientframe);
      break;
    case 4: /* Background Colors */
      gtk_widget_show(menuvars.bgframe);
      break;
    case 6: /* Light */
      gtk_widget_show(menuvars.lightframe);
      break;
    case 7: /* Objects */
      gtk_widget_show(menuvars.objectframe);
      break;
  }
}

static GtkItemFactoryEntry menu_items[] = {
  { "/_File",		NULL,		NULL,	0, "<Branch>" },
  { "/File/_Open",	"<control>O",	OpenNew,	0, NULL },
  { "/File/_Save",	"<control>S",	Snapshot,	0, NULL },
  { "/File/Save _As",	"<control>A",	SaveSnap,	0, NULL },
  { "/File/_Write rc",	"<alt>W",	Write_rc,	0, NULL },
  { "/File/sep1",	NULL,		NULL,	0, "<Separator>" },
  { "/File/Quit",	"<control>Q",	Menu_Quit,0, NULL },
  { "/_View",		NULL,		NULL,	0, "<Branch>" },
  { "/View/_Frame frequency",	NULL,	Menu_Frame,	0, NULL },
  { "/View/_Movie",	"<alt>M",	Menu_Frame,	1, NULL },
  { "/View/_Eye",	NULL,		Menu_Frame,	2, NULL },
  { "/View/_Orientation",	NULL,	Menu_Frame,	3, NULL },
  { "/View/_Backgound",	"<alt>B",	Menu_Frame,	4, NULL },
  { "/View/_Light",	"<alt>L",	Menu_Frame,	6, NULL },
  { "/View/Ob_jects",	"<alt>J",	Menu_Frame,	7, NULL },
  { "/_Control",	NULL,		NULL,	0, "<Branch>" },
  { "/Control/_Run",	"<alt>R",	Menu_Run,	0, NULL },
  { "/Control/_Stop",	"<alt>S",	Menu_Run,	1, NULL },
  { "/Control/_Pause",	"<alt>P",	Pause_Signal,	0, NULL },
  { "/Control/Play _Direction",	NULL,	NULL,	0, "<Branch>" },
  { "/Control/Play Direction/_Forward cont.",
    			NULL,		Set_PlayDir,	0, NULL },
  { "/Control/Play Direction/F_orward",
    			NULL,		Set_PlayDir,	1, NULL },
  { "/Control/Play Direction/_Reverse cont.",
    			NULL,		Set_PlayDir,	2, NULL },
  { "/Control/Play Direction/R_everse",
    			NULL,		Set_PlayDir,	3, NULL },
  { "/_Set",		NULL,		NULL,	0, "<Branch>" },
  { "/Set/_Enlargement factor",
    			NULL ,		Menu_Edit,	5, NULL },
  { "/Set/Fast _play mult.",
    			NULL ,		Menu_Edit,	1, NULL },
  { "/Set/_Framedt max",
    			NULL ,		Menu_Edit,	0, NULL },
  { "/Set/_Min Sphere res.",
    			NULL ,		Menu_Edit,	3, NULL },
  { "/Set/Framedt limit for Sphere _res. adj.",
    			NULL ,		Menu_Edit,	4, NULL },
  { "/Set/_XYZ factor",
    			NULL ,		Menu_Edit,	2, NULL },
  { "/Set/_Key step",
    			NULL ,		Menu_Edit,	6, NULL },
  { "/_Help",		NULL,		NULL,	0, "<Branch>" },
  { "/_Help/_About",	NULL,		About,	0,	NULL },
  { "/_Help/Little _Help",	NULL,	Help,	0,	NULL },
};


void get_main_menu(GtkWidget *window,GtkWidget **menubar,
					GtkAccelGroup *accel_group)
{
  GtkItemFactory *item_factory;
  gint nmenu_items = sizeof (menu_items) / sizeof (menu_items[0]);


  /* This function initializes the item factory.
     Param 1: The type of menu - can be GTK_TYPE_MENU_BAR, GTK_TYPE_MENU,
              or GTK_TYPE_OPTION_MENU.
     Param 2: The path of the menuvars.
     Param 3: A pointer to a gtk_accel_group.  The item factory sets up
              the accelerator table while generating menus.
  */

  item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, "<main>", 
				       accel_group);

  /* This function generates the menu items. Pass the item factory,
     the number of items in the array, the array itself, and any
     callback data for the the menu items. */
  gtk_item_factory_create_items (item_factory, nmenu_items, menu_items, NULL);

  /* Attach the new accelerator group to the window. */
  gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);

  /* Finally, return the actual menu bar created by the item factory. */ 
  if (menubar) {
    *menubar = gtk_item_factory_get_widget (item_factory, "<main>");
  }
  menuvars.openfileisup=0;
}

